@@ -2,14 +2,15 @@ from UM.Scene.SceneNodeDecorator import SceneNodeDecorator
 from cura.Scene.CuraSceneNode import CuraSceneNode
 
 
-##  Make a SceneNode build plate aware CuraSceneNode objects all have this decorator.
 class BuildPlateDecorator(SceneNodeDecorator):
-    def __init__(self, build_plate_number = -1):
+    """Make a SceneNode build plate aware CuraSceneNode objects all have this decorator."""
+
+    def __init__(self, build_plate_number: int = -1) -> None:
         super().__init__()
-        self._build_plate_number = None
+        self._build_plate_number = build_plate_number
         self.setBuildPlateNumber(build_plate_number)
 
-    def setBuildPlateNumber(self, nr):
+    def setBuildPlateNumber(self, nr: int) -> None:
         # Make sure that groups are set correctly
         # setBuildPlateForSelection in CuraActions makes sure that no single childs are set.
         self._build_plate_number = nr
@@ -19,7 +20,7 @@ class BuildPlateDecorator(SceneNodeDecorator):
             for child in self._node.getChildren():
                 child.callDecoration("setBuildPlateNumber", nr)
 
-    def getBuildPlateNumber(self):
+    def getBuildPlateNumber(self) -> int:
         return self._build_plate_number
 
     def __deepcopy__(self, memo):
