@@ -1,4 +1,4 @@
-# Copyright (c) 2019 Ultimaker B.V.
+# Copyright (c) 2021 Ultimaker B.V.
 # Cura is released under the terms of the LGPLv3 or higher.
 
 import os
@@ -32,6 +32,10 @@ from cura.Machines.ContainerTree import ContainerTree
 from cura.ReaderWriters.ProfileReader import NoProfileException, ProfileReader
 
 from UM.i18n import i18nCatalog
+from .DatabaseHandlers.IntentDatabaseHandler import IntentDatabaseHandler
+from .DatabaseHandlers.QualityDatabaseHandler import QualityDatabaseHandler
+from .DatabaseHandlers.VariantDatabaseHandler import VariantDatabaseHandler
+
 catalog = i18nCatalog("cura")
 
 
@@ -44,6 +48,10 @@ class CuraContainerRegistry(ContainerRegistry):
         # is added, we check to see if an extruder stack needs to be added.
         self.containerAdded.connect(self._onContainerAdded)
 
+        self._database_handlers["variant"] = VariantDatabaseHandler()
+        self._database_handlers["quality"] = QualityDatabaseHandler()
+        self._database_handlers["intent"] = IntentDatabaseHandler()
+
     @override(ContainerRegistry)
     def addContainer(self, container: ContainerInterface) -> bool:
         """Overridden from ContainerRegistry
@@ -141,20 +149,29 @@ class CuraContainerRegistry(ContainerRegistry):
             success = profile_writer.write(file_name, container_list)
         except Exception as e:
             Logger.log("e", "Failed to export profile to %s: %s", file_name, str(e))
-            m = Message(catalog.i18nc("@info:status Don't translate the XML tags <filename> or <message>!", "Failed to export profile to <filename>{0}</filename>: <message>{1}</message>", file_name, str(e)),
+            m = Message(catalog.i18nc("@info:status Don't translate the XML tags <filename> or <message>!",
+                                      "Failed to export profile to <filename>{0}</filename>: <message>{1}</message>",
+                                      file_name, str(e)),
                         lifetime = 0,
-                        title = catalog.i18nc("@info:title", "Error"))
+                        title = catalog.i18nc("@info:title", "Error"),
+                        message_type = Message.MessageType.ERROR)
             m.show()
             return False
         if not success:
             Logger.log("w", "Failed to export profile to %s: Writer plugin reported failure.", file_name)
-            m = Message(catalog.i18nc("@info:status Don't translate the XML tag <filename>!", "Failed to export profile to <filename>{0}</filename>: Writer plugin reported failure.", file_name),
+            m = Message(catalog.i18nc("@info:status Don't translate the XML tag <filename>!",
+                                      "Failed to export profile to <filename>{0}</filename>: Writer plugin reported failure.",
+                                      file_name),
                         lifetime = 0,
-                        title = catalog.i18nc("@info:title", "Error"))
+                        title = catalog.i18nc("@info:title", "Error"),
+                        message_type = Message.MessageType.ERROR)
             m.show()
             return False
-        m = Message(catalog.i18nc("@info:status Don't translate the XML tag <filename>!", "Exported profile to <filename>{0}</filename>", file_name),
-                    title = catalog.i18nc("@info:title", "Export succeeded"))
+        m = Message(catalog.i18nc("@info:status Don't translate the XML tag <filename>!",
+                                  "Exported profile to <filename>{0}</filename>",
+                                  file_name),
+                    title = catalog.i18nc("@info:title", "Export succeeded"),
+                    message_type = Message.MessageType.POSITIVE)
         m.show()
         return True
 
@@ -381,9 +398,10 @@ class CuraContainerRegistry(ContainerRegistry):
             if profile_count > 1:
                 continue
             # Only one profile found, this should not ever be the case, so that profile needs to be removed!
-            Logger.log("d", "Found an invalid quality_changes profile with the name %s. Going to remove that now", profile_name)
             invalid_quality_changes = ContainerRegistry.getInstance().findContainersMetadata(name=profile_name)
-            self.removeContainer(invalid_quality_changes[0]["id"])
+            if invalid_quality_changes:
+                Logger.log("d", "Found an invalid quality_changes profile with the name %s. Going to remove that now", profile_name)
+                self.removeContainer(invalid_quality_changes[0]["id"])
 
     @override(ContainerRegistry)
     def _isMetadataValid(self, metadata: Optional[Dict[str, Any]]) -> bool:
@@ -400,7 +418,9 @@ class CuraContainerRegistry(ContainerRegistry):
         try:
             if int(metadata["setting_version"]) != cura.CuraApplication.CuraApplication.SettingVersion:
                 return False
-        except ValueError: #Not parsable as int.
+        except ValueError:  # Not parsable as int.
+            return False
+        except TypeError:  # Expecting string input here, not e.g. list or anything.
             return False
         return True
 
@@ -440,7 +460,7 @@ class CuraContainerRegistry(ContainerRegistry):
 
         global_stack = cura.CuraApplication.CuraApplication.getInstance().getGlobalContainerStack()
         if not global_stack:
-            return False, catalog.i18nc("@info:status", "Global stack is missing.")
+            return False, catalog.i18nc("@info:status", "There is no active printer yet.")
 
         definition_id = ContainerTree.getInstance().machines[global_stack.definition.getId()].quality_definition
         profile.setDefinition(definition_id)
