@@ -1,4 +1,4 @@
-# Copyright (c) 2016 Ultimaker B.V.
+# Copyright (c) 2021 Ultimaker B.V.
 # Cura is released under the terms of the LGPLv3 or higher.
 
 from PyQt5.QtCore import pyqtSlot, pyqtProperty, QObject, pyqtSignal, QRegExp
@@ -10,17 +10,20 @@ from UM.Resources import Resources
 from UM.Settings.ContainerRegistry import ContainerRegistry
 from UM.Settings.InstanceContainer import InstanceContainer
 
-##  Are machine names valid?
-#
-#   Performs checks based on the length of the name.
+
 class MachineNameValidator(QObject):
+    """Are machine names valid?
+
+    Performs checks based on the length of the name.
+    """
+
     def __init__(self, parent = None):
         super().__init__(parent)
 
         #Compute the validation regex for printer names. This is limited by the maximum file name length.
         try:
             filename_max_length = os.statvfs(Resources.getDataStoragePath()).f_namemax
-        except AttributeError: #Doesn't support statvfs. Probably because it's not a Unix system.
+        except (AttributeError, EnvironmentError):  # Doesn't support statvfs. Probably because it's not a Unix system. Or perhaps there is no permission or it doesn't exist.
             filename_max_length = 255 #Assume it's Windows on NTFS.
         machine_name_max_length = filename_max_length - len("_current_settings.") - len(ContainerRegistry.getMimeTypeForContainer(InstanceContainer).preferredSuffix)
         # Characters that urllib.parse.quote_plus escapes count for 12! So now
@@ -28,16 +31,17 @@ class MachineNameValidator(QObject):
         # special character, and that up to [machine_name_max_length / 12] times.
         maximum_special_characters = int(machine_name_max_length / 12)
         unescaped = r"[a-zA-Z0-9_\-\.\/]"
-        self.machine_name_regex = r"^((" + unescaped + "){0,12}|.){0," + str(maximum_special_characters) + r"}$"
+        self.machine_name_regex = r"^[^\.]((" + unescaped + "){0,12}|.){0," + str(maximum_special_characters) + r"}$"
 
     validationChanged = pyqtSignal()
 
-    ##  Check if a specified machine name is allowed.
-    #
-    #   \param name The machine name to check.
-    #   \return ``QValidator.Invalid`` if it's disallowed, or
-    #   ``QValidator.Acceptable`` if it's allowed.
     def validate(self, name):
+        """Check if a specified machine name is allowed.
+
+        :param name: The machine name to check.
+        :return: ``QValidator.Invalid`` if it's disallowed, or ``QValidator.Acceptable`` if it's allowed.
+        """
+
         #Check for file name length of the current settings container (which is the longest file we're saving with the name).
         try:
             filename_max_length = os.statvfs(Resources.getDataStoragePath()).f_namemax
@@ -50,9 +54,10 @@ class MachineNameValidator(QObject):
 
         return QValidator.Acceptable #All checks succeeded.
 
-    ##  Updates the validation state of a machine name text field.
     @pyqtSlot(str)
     def updateValidation(self, new_name):
+        """Updates the validation state of a machine name text field."""
+
         is_valid = self.validate(new_name)
         if is_valid == QValidator.Acceptable:
             self.validation_regex = "^.*$" #Matches anything.
