@@ -7,7 +7,7 @@ SCRIPT_DIR="$( cd "$( dirname "${BASH_SOURCE[0]}" )" >/dev/null 2>&1 && pwd )"
 PROJECT_DIR="$( cd "${SCRIPT_DIR}/.." && pwd )"
 
 # Make sure that environment variables are set properly
-source /opt/rh/devtoolset-7/enable
+source /opt/rh/devtoolset-8/enable
 export PATH="${CURA_BUILD_ENV_PATH}/bin:${PATH}"
 export PKG_CONFIG_PATH="${CURA_BUILD_ENV_PATH}/lib/pkgconfig:${PKG_CONFIG_PATH}"
 
@@ -50,7 +50,7 @@ do
     echo "Found Uranium branch [${URANIUM_BRANCH}]."
     break
   else
-    echo "Could not find Uranium banch [${URANIUM_BRANCH}], try next."
+    echo "Could not find Uranium branch [${URANIUM_BRANCH}], try next."
   fi
 done
 
