@@ -34,6 +34,9 @@ class DrivePluginExtension(QObject, Extension):
     # Signal emitted when preferences changed (like auto-backup).
     preferencesChanged = pyqtSignal()
 
+    # Signal emitted when the id of the backup-to-be-restored is changed
+    backupIdBeingRestoredChanged = pyqtSignal(arguments = ["backup_id_being_restored"])
+
     DATE_FORMAT = "%d/%m/%Y %H:%M:%S"
 
     def __init__(self) -> None:
@@ -45,6 +48,7 @@ class DrivePluginExtension(QObject, Extension):
         self._backups = []  # type: List[Dict[str, Any]]
         self._is_restoring_backup = False
         self._is_creating_backup = False
+        self._backup_id_being_restored = ""
 
         # Initialize services.
         preferences = CuraApplication.getInstance().getPreferences()
@@ -52,6 +56,7 @@ class DrivePluginExtension(QObject, Extension):
 
         # Attach signals.
         CuraApplication.getInstance().getCuraAPI().account.loginStateChanged.connect(self._onLoginStateChanged)
+        CuraApplication.getInstance().applicationShuttingDown.connect(self._onApplicationShuttingDown)
         self._drive_api_service.restoringStateChanged.connect(self._onRestoringStateChanged)
         self._drive_api_service.creatingStateChanged.connect(self._onCreatingStateChanged)
 
@@ -75,6 +80,10 @@ class DrivePluginExtension(QObject, Extension):
         if self._drive_window:
             self._drive_window.show()
 
+    def _onApplicationShuttingDown(self):
+        if self._drive_window:
+            self._drive_window.hide()
+
     def _autoBackup(self) -> None:
         preferences = CuraApplication.getInstance().getPreferences()
         if preferences.getValue(Settings.AUTO_BACKUP_ENABLED_PREFERENCE_KEY) and self._isLastBackupTooLongAgo():
@@ -100,17 +109,22 @@ class DrivePluginExtension(QObject, Extension):
         if logged_in:
             self.refreshBackups()
 
-    def _onRestoringStateChanged(self, is_restoring: bool = False, error_message: str = None) -> None:
+    def _onRestoringStateChanged(self, is_restoring: bool = False, error_message: Optional[str] = None) -> None:
         self._is_restoring_backup = is_restoring
         self.restoringStateChanged.emit()
         if error_message:
-            Message(error_message, title = catalog.i18nc("@info:title", "Backup")).show()
+            self.backupIdBeingRestored = ""
+            Message(error_message,
+                    title = catalog.i18nc("@info:title", "Backup"),
+                    message_type = Message.MessageType.ERROR).show()
 
     def _onCreatingStateChanged(self, is_creating: bool = False, error_message: str = None) -> None:
         self._is_creating_backup = is_creating
         self.creatingStateChanged.emit()
         if error_message:
-            Message(error_message, title = catalog.i18nc("@info:title", "Backup")).show()
+            Message(error_message,
+                    title = catalog.i18nc("@info:title", "Backup"),
+                    message_type = Message.MessageType.ERROR).show()
         else:
             self._storeBackupDate()
         if not is_creating and not error_message:
@@ -152,6 +166,7 @@ class DrivePluginExtension(QObject, Extension):
         for backup in self._backups:
             if backup.get("backup_id") == backup_id:
                 self._drive_api_service.restoreBackup(backup)
+                self.setBackupIdBeingRestored(backup_id)
                 return
         Logger.log("w", "Unable to find backup with the ID %s", backup_id)
 
@@ -166,3 +181,12 @@ class DrivePluginExtension(QObject, Extension):
     def _backupDeletedCallback(self, success: bool):
         if success:
             self.refreshBackups()
+
+    def setBackupIdBeingRestored(self, backup_id_being_restored: str) -> None:
+        if backup_id_being_restored != self._backup_id_being_restored:
+            self._backup_id_being_restored = backup_id_being_restored
+            self.backupIdBeingRestoredChanged.emit()
+
+    @pyqtProperty(str, fset = setBackupIdBeingRestored, notify = backupIdBeingRestoredChanged)
+    def backupIdBeingRestored(self) -> str:
+        return self._backup_id_being_restored
