@@ -1,4 +1,4 @@
-# Copyright (c) 2020 Ultimaker B.V.
+# Copyright (c) 2021 Ultimaker B.V.
 # Cura is released under the terms of the LGPLv3 or higher.
 
 import numpy
@@ -8,12 +8,14 @@ import time
 from typing import Any, cast, Dict, List, Optional, Set
 import re
 import Arcus #For typing.
+from PyQt5.QtCore import QCoreApplication
 
 from UM.Job import Job
 from UM.Logger import Logger
 from UM.Scene.SceneNode import SceneNode
 from UM.Settings.ContainerStack import ContainerStack #For typing.
 from UM.Settings.InstanceContainer import InstanceContainer
+from UM.Settings.Interfaces import ContainerInterface
 from UM.Settings.SettingDefinition import SettingDefinition
 from UM.Settings.SettingRelation import SettingRelation #For typing.
 
@@ -42,8 +44,9 @@ class StartJobResult(IntEnum):
     ObjectsWithDisabledExtruder = 8
 
 
-##  Formatter class that handles token expansion in start/end gcode
 class GcodeStartEndFormatter(Formatter):
+    """Formatter class that handles token expansion in start/end gcode"""
+
     def __init__(self, default_extruder_nr: int = -1) -> None:
         super().__init__()
         self._default_extruder_nr = default_extruder_nr
@@ -84,8 +87,9 @@ class GcodeStartEndFormatter(Formatter):
         return value
 
 
-##  Job class that builds up the message of scene data to send to CuraEngine.
 class StartSliceJob(Job):
+    """Job class that builds up the message of scene data to send to CuraEngine."""
+
     def __init__(self, slice_message: Arcus.PythonMessage) -> None:
         super().__init__()
 
@@ -102,9 +106,10 @@ class StartSliceJob(Job):
     def setBuildPlate(self, build_plate_number: int) -> None:
         self._build_plate_number = build_plate_number
 
-    ##  Check if a stack has any errors.
-    ##  returns true if it has errors, false otherwise.
     def _checkStackForErrors(self, stack: ContainerStack) -> bool:
+        """Check if a stack has any errors."""
+
+        """returns true if it has errors, false otherwise."""
 
         top_of_stack = cast(InstanceContainer, stack.getTop())  # Cache for efficiency.
         changed_setting_keys = top_of_stack.getAllKeys()
@@ -118,6 +123,9 @@ class StartSliceJob(Job):
             Job.yieldThread()
 
         for changed_setting_key in changed_setting_keys:
+            if not stack.getProperty(changed_setting_key, "enabled"):
+                continue
+
             validation_state = stack.getProperty(changed_setting_key, "validationState")
 
             if validation_state is None:
@@ -134,8 +142,9 @@ class StartSliceJob(Job):
 
         return False
 
-    ##  Runs the job that initiates the slicing.
     def run(self) -> None:
+        """Runs the job that initiates the slicing."""
+
         if self._build_plate_number is None:
             self.setResult(StartJobResult.Error)
             return
@@ -189,7 +198,7 @@ class StartSliceJob(Job):
         # Remove old layer data.
         for node in DepthFirstIterator(self._scene.getRoot()):
             if node.callDecoration("getLayerData") and node.callDecoration("getBuildPlateNumber") == self._build_plate_number:
-                # Singe we walk through all nodes in the scene, they always have a parent.
+                # Since we walk through all nodes in the scene, they always have a parent.
                 cast(SceneNode, node.getParent()).removeChild(node)
                 break
 
@@ -199,10 +208,6 @@ class StartSliceJob(Job):
             for node in OneAtATimeIterator(self._scene.getRoot()):
                 temp_list = []
 
-                # Node can't be printed, so don't bother sending it.
-                if getattr(node, "_outside_buildarea", False):
-                    continue
-
                 # Filter on current build plate
                 build_plate_number = node.callDecoration("getBuildPlateNumber")
                 if build_plate_number is not None and build_plate_number != self._build_plate_number:
@@ -251,7 +256,7 @@ class StartSliceJob(Job):
         global_stack = CuraApplication.getInstance().getGlobalContainerStack()
         if not global_stack:
             return
-        extruders_enabled = {position: stack.isEnabled for position, stack in global_stack.extruders.items()}
+        extruders_enabled = [stack.isEnabled for stack in global_stack.extruderList]
         filtered_object_groups = []
         has_model_with_disabled_extruders = False
         associated_disabled_extruders = set()
@@ -261,7 +266,7 @@ class StartSliceJob(Job):
             for node in group:
                 # Only check if the printing extruder is enabled for printing meshes
                 is_non_printing_mesh = node.callDecoration("evaluateIsNonPrintingMesh")
-                extruder_position = node.callDecoration("getActiveExtruderPosition")
+                extruder_position = int(node.callDecoration("getActiveExtruderPosition"))
                 if not is_non_printing_mesh and not extruders_enabled[extruder_position]:
                     skip_group = True
                     has_model_with_disabled_extruders = True
@@ -271,8 +276,8 @@ class StartSliceJob(Job):
 
         if has_model_with_disabled_extruders:
             self.setResult(StartJobResult.ObjectsWithDisabledExtruder)
-            associated_disabled_extruders = {str(c) for c in sorted([int(p) + 1 for p in associated_disabled_extruders])}
-            self.setMessage(", ".join(associated_disabled_extruders))
+            associated_disabled_extruders = {p + 1 for p in associated_disabled_extruders}
+            self.setMessage(", ".join(map(str, sorted(associated_disabled_extruders))))
             return
 
         # There are cases when there is nothing to slice. This can happen due to one at a time slicing not being
@@ -338,24 +343,32 @@ class StartSliceJob(Job):
     def setIsCancelled(self, value: bool):
         self._is_cancelled = value
 
-    ##  Creates a dictionary of tokens to replace in g-code pieces.
-    #
-    #   This indicates what should be replaced in the start and end g-codes.
-    #   \param stack The stack to get the settings from to replace the tokens
-    #   with.
-    #   \return A dictionary of replacement tokens to the values they should be
-    #   replaced with.
     def _buildReplacementTokens(self, stack: ContainerStack) -> Dict[str, Any]:
+        """Creates a dictionary of tokens to replace in g-code pieces.
+
+        This indicates what should be replaced in the start and end g-codes.
+        :param stack: The stack to get the settings from to replace the tokens with.
+        :return: A dictionary of replacement tokens to the values they should be replaced with.
+        """
+
         result = {}
         for key in stack.getAllKeys():
-            value = stack.getProperty(key, "value")
-            result[key] = value
+            result[key] = stack.getProperty(key, "value")
             Job.yieldThread()
 
-        result["print_bed_temperature"] = result["material_bed_temperature"] # Renamed settings.
+        # Material identification in addition to non-human-readable GUID
+        result["material_id"] = stack.material.getMetaDataEntry("base_file", "")
+        result["material_type"] = stack.material.getMetaDataEntry("material", "")
+        result["material_name"] = stack.material.getMetaDataEntry("name", "")
+        result["material_brand"] = stack.material.getMetaDataEntry("brand", "")
+
+        # Renamed settings.
+        result["print_bed_temperature"] = result["material_bed_temperature"]
         result["print_temperature"] = result["material_print_temperature"]
         result["travel_speed"] = result["speed_travel"]
-        result["time"] = time.strftime("%H:%M:%S") #Some extra settings.
+
+        #Some extra settings.
+        result["time"] = time.strftime("%H:%M:%S")
         result["date"] = time.strftime("%d-%m-%Y")
         result["day"] = ["Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat"][int(time.strftime("%w"))]
         result["initial_extruder_nr"] = CuraApplication.getInstance().getExtruderManager().getInitialExtruderNr()
@@ -369,14 +382,18 @@ class StartSliceJob(Job):
         self._all_extruders_settings = {
             "-1": self._buildReplacementTokens(global_stack)
         }
+        QCoreApplication.processEvents()  # Ensure that the GUI does not freeze.
         for extruder_stack in ExtruderManager.getInstance().getActiveExtruderStacks():
             extruder_nr = extruder_stack.getProperty("extruder_nr", "value")
             self._all_extruders_settings[str(extruder_nr)] = self._buildReplacementTokens(extruder_stack)
+            QCoreApplication.processEvents()  # Ensure that the GUI does not freeze.
 
-    ##  Replace setting tokens in a piece of g-code.
-    #   \param value A piece of g-code to replace tokens in.
-    #   \param default_extruder_nr Stack nr to use when no stack nr is specified, defaults to the global stack
     def _expandGcodeTokens(self, value: str, default_extruder_nr: int = -1) -> str:
+        """Replace setting tokens in a piece of g-code.
+
+        :param value: A piece of g-code to replace tokens in.
+        :param default_extruder_nr: Stack nr to use when no stack nr is specified, defaults to the global stack
+        """
         if not self._all_extruders_settings:
             self._cacheAllExtruderSettings()
 
@@ -392,8 +409,9 @@ class StartSliceJob(Job):
             Logger.logException("w", "Unable to do token replacement on start/end g-code")
             return str(value)
 
-    ##  Create extruder message from stack
     def _buildExtruderMessage(self, stack: ContainerStack) -> None:
+        """Create extruder message from stack"""
+
         message = self._slice_message.addRepeatedMessage("extruders")
         message.id = int(stack.getMetaDataEntry("position"))
         if not self._all_extruders_settings:
@@ -413,20 +431,27 @@ class StartSliceJob(Job):
         settings["machine_extruder_start_code"] = self._expandGcodeTokens(settings["machine_extruder_start_code"], extruder_nr)
         settings["machine_extruder_end_code"] = self._expandGcodeTokens(settings["machine_extruder_end_code"], extruder_nr)
 
+        global_definition = cast(ContainerInterface, cast(ContainerStack, stack.getNextStack()).getBottom())
+        own_definition = cast(ContainerInterface, stack.getBottom())
+
         for key, value in settings.items():
             # Do not send settings that are not settable_per_extruder.
-            if not stack.getProperty(key, "settable_per_extruder"):
-                continue
+            # Since these can only be set in definition files, we only have to ask there.
+            if not global_definition.getProperty(key, "settable_per_extruder") and \
+                    not own_definition.getProperty(key, "settable_per_extruder"):
+                    continue
             setting = message.getMessage("settings").addRepeatedMessage("settings")
             setting.name = key
             setting.value = str(value).encode("utf-8")
             Job.yieldThread()
 
-    ##  Sends all global settings to the engine.
-    #
-    #   The settings are taken from the global stack. This does not include any
-    #   per-extruder settings or per-object settings.
     def _buildGlobalSettingsMessage(self, stack: ContainerStack) -> None:
+        """Sends all global settings to the engine.
+
+        The settings are taken from the global stack. This does not include any
+        per-extruder settings or per-object settings.
+        """
+
         if not self._all_extruders_settings:
             self._cacheAllExtruderSettings()
 
@@ -442,14 +467,13 @@ class StartSliceJob(Job):
         bed_temperature_settings = ["material_bed_temperature", "material_bed_temperature_layer_0"]
         pattern = r"\{(%s)(,\s?\w+)?\}" % "|".join(bed_temperature_settings) # match {setting} as well as {setting, extruder_nr}
         settings["material_bed_temp_prepend"] = re.search(pattern, start_gcode) == None
-        print_temperature_settings = ["material_print_temperature", "material_print_temperature_layer_0", "default_material_print_temperature", "material_initial_print_temperature", "material_final_print_temperature", "material_standby_temperature"]
+        print_temperature_settings = ["material_print_temperature", "material_print_temperature_layer_0", "default_material_print_temperature", "material_initial_print_temperature", "material_final_print_temperature", "material_standby_temperature", "print_temperature"]
         pattern = r"\{(%s)(,\s?\w+)?\}" % "|".join(print_temperature_settings) # match {setting} as well as {setting, extruder_nr}
-        settings["material_print_temp_prepend"] = re.search(pattern, start_gcode) == None
+        settings["material_print_temp_prepend"] = re.search(pattern, start_gcode) is None
+
         # Replace the setting tokens in start and end g-code.
         # Use values from the first used extruder by default so we get the expected temperatures
-        initial_extruder_stack = CuraApplication.getInstance().getExtruderManager().getUsedExtruderStacks()[0]
-        initial_extruder_nr = initial_extruder_stack.getProperty("extruder_nr", "value")
-
+        initial_extruder_nr = CuraApplication.getInstance().getExtruderManager().getInitialExtruderNr()
         settings["machine_start_gcode"] = self._expandGcodeTokens(settings["machine_start_gcode"], initial_extruder_nr)
         settings["machine_end_gcode"] = self._expandGcodeTokens(settings["machine_end_gcode"], initial_extruder_nr)
 
@@ -460,15 +484,16 @@ class StartSliceJob(Job):
             setting_message.value = str(value).encode("utf-8")
             Job.yieldThread()
 
-    ##  Sends for some settings which extruder they should fallback to if not
-    #   set.
-    #
-    #   This is only set for settings that have the limit_to_extruder
-    #   property.
-    #
-    #   \param stack The global stack with all settings, from which to read the
-    #   limit_to_extruder property.
     def _buildGlobalInheritsStackMessage(self, stack: ContainerStack) -> None:
+        """Sends for some settings which extruder they should fallback to if not set.
+
+        This is only set for settings that have the limit_to_extruder
+        property.
+
+        :param stack: The global stack with all settings, from which to read the
+            limit_to_extruder property.
+        """
+
         for key in stack.getAllKeys():
             extruder_position = int(round(float(stack.getProperty(key, "limit_to_extruder"))))
             if extruder_position >= 0:  # Set to a specific extruder.
@@ -477,10 +502,13 @@ class StartSliceJob(Job):
                 setting_extruder.extruder = extruder_position
             Job.yieldThread()
 
-    ##  Check if a node has per object settings and ensure that they are set correctly in the message
-    #   \param node Node to check.
-    #   \param message object_lists message to put the per object settings in
     def _handlePerObjectSettings(self, node: CuraSceneNode, message: Arcus.PythonMessage):
+        """Check if a node has per object settings and ensure that they are set correctly in the message
+
+        :param node: Node to check.
+        :param message: object_lists message to put the per object settings in
+        """
+
         stack = node.callDecoration("getStack")
 
         # Check if the node has a stack attached to it and the stack has any settings in the top container.
@@ -516,10 +544,13 @@ class StartSliceJob(Job):
 
             Job.yieldThread()
 
-    ##  Recursive function to put all settings that require each other for value changes in a list
-    #   \param relations_set Set of keys of settings that are influenced
-    #   \param relations list of relation objects that need to be checked.
     def _addRelations(self, relations_set: Set[str], relations: List[SettingRelation]):
+        """Recursive function to put all settings that require each other for value changes in a list
+
+        :param relations_set: Set of keys of settings that are influenced
+        :param relations: list of relation objects that need to be checked.
+        """
+
         for relation in filter(lambda r: r.role == "value" or r.role == "limit_to_extruder", relations):
             if relation.type == RelationType.RequiresTarget:
                 continue
