@@ -1,4 +1,4 @@
-# Copyright (c) 2020 Ultimaker B.V.
+# Copyright (c) 2021 Ultimaker B.V.
 # Cura is released under the terms of the LGPLv3 or higher.
 
 import math
@@ -153,7 +153,7 @@ class FlavorParser:
         Af = (self._filament_diameter / 2) ** 2 * numpy.pi
         # Length of the extruded filament
         de = current_extrusion - previous_extrusion
-        # Volumne of the extruded filament
+        # Volume of the extruded filament
         dVe = de * Af
         # Length of the printed line
         dX = numpy.sqrt((current_point[0] - previous_point[0])**2 + (current_point[2] - previous_point[2])**2)
@@ -198,7 +198,7 @@ class FlavorParser:
 
             # Only when extruding we can determine the latest known "layer height" which is the difference in height between extrusions
             # Also, 1.5 is a heuristic for any priming or whatsoever, we skip those.
-            if z > self._previous_z and (z - self._previous_z < 1.5):
+            if z > self._previous_z and (z - self._previous_z < 1.5) and (params.x is not None or params.y is not None):
                 self._current_layer_thickness = z - self._previous_z # allow a tiny overlap
                 self._previous_z = z
         elif self._previous_extrusion_value > e[self._extruder_number]:
@@ -428,7 +428,7 @@ class FlavorParser:
 
             G = self._getInt(line, "G")
             if G is not None:
-                # When find a movement, the new posistion is calculated and added to the current_path, but
+                # When find a movement, the new position is calculated and added to the current_path, but
                 # don't need to create a polygon until the end of the layer
                 current_position = self.processGCode(G, line, current_position, current_path)
                 continue
@@ -500,7 +500,8 @@ class FlavorParser:
                 "@info:generic",
                 "Make sure the g-code is suitable for your printer and printer configuration before sending the file to it. The g-code representation may not be accurate."),
                 lifetime=0,
-                title = catalog.i18nc("@info:title", "G-code Details"))
+                title = catalog.i18nc("@info:title", "G-code Details"),
+                message_type = Message.MessageType.WARNING)
             caution_message.show()
 
         # The "save/print" button's state is bound to the backend state.
