@@ -10,10 +10,10 @@ WARNING This script has never been tested with several extruders
 from ..Script import Script
 import numpy as np
 from UM.Logger import Logger
-from UM.Application import Application
 import re
 from cura.Settings.ExtruderManager import ExtruderManager
 
+
 def _getValue(line, key, default=None):
     """
     Convenience function that finds the value in a line of g-code.
@@ -30,12 +30,13 @@ def _getValue(line, key, default=None):
         return default
     return float(number.group(0))
 
+
 class GCodeStep():
     """
     Class to store the current value of each G_Code parameter
     for any G-Code step
     """
-    def __init__(self, step, in_relative_movement: bool = False):
+    def __init__(self, step, in_relative_movement: bool = False) -> None:
         self.step = step
         self.step_x = 0
         self.step_y = 0
@@ -85,7 +86,7 @@ class GCodeStep():
 
 
 # Execution part of the stretch plugin
-class Stretcher():
+class Stretcher:
     """
     Execution part of the stretch algorithm
     """
@@ -194,7 +195,7 @@ class Stretcher():
         i.e. it is a travel move
         """
         if i_pos == 0:
-            return True # Begining a layer always breaks filament (for simplicity)
+            return True # Beginning a layer always breaks filament (for simplicity)
         step = layer_steps[i_pos]
         prev_step = layer_steps[i_pos - 1]
         if step.step_e != prev_step.step_e:
@@ -207,7 +208,6 @@ class Stretcher():
             return False
         return True # New sequence
 
-
     def processLayer(self, layer_steps):
         """
         Computes the new coordinates of g-code steps
@@ -289,9 +289,15 @@ class Stretcher():
                 self.layergcode = self.layergcode + sout + "\n"
                 ipos = ipos + 1
             else:
+                # The command is intended to be passed through unmodified via
+                # the comment field. In the case of an extruder only move, though,
+                # the extruder and potentially the feed rate are modified.
+                # We need to update self.outpos accordingly so that subsequent calls
+                # to stepToGcode() knows about the extruder and feed rate change.
+                self.outpos.step_e = layer_steps[i].step_e
+                self.outpos.step_f = layer_steps[i].step_f
                 self.layergcode = self.layergcode + layer_steps[i].comment + "\n"
 
-
     def workOnSequence(self, orig_seq, modif_seq):
         """
         Computes new coordinates for a sequence
