@@ -14,5 +14,5 @@ class PrintJobUploadErrorMessage(Message):
         super().__init__(
             text = message or I18N_CATALOG.i18nc("@info:text", "Could not upload the data to the printer."),
             title = I18N_CATALOG.i18nc("@info:title", "Network error"),
-            lifetime = 10
+            message_type=Message.MessageType.ERROR
         )
