@@ -1,3 +1,6 @@
+# Copyright (c) 2021 Ultimaker B.V.
+# Cura is released under the terms of the LGPLv3 or higher.
+
 import time
 from unittest.mock import MagicMock, patch
 
@@ -38,7 +41,7 @@ def test_post():
     mocked_network_manager = MagicMock()
     output_device._manager = mocked_network_manager
 
-    # Create a fake reply (we cant use a QReply, since those are abstract C++)
+    # Create a fake reply (we can't use a QReply, since those are abstract C++)
     reply = MagicMock()
     reply.operation = MagicMock(return_value=QNetworkAccessManager.PostOperation)
     reply.url = MagicMock(return_value=QUrl("127.0.0.1"))
@@ -47,7 +50,7 @@ def test_post():
     mocked_callback_handler = MagicMock()
     output_device.post("whatever", "omgzomg", on_finished = mocked_callback_handler.onFinished)
 
-    # So we now fake that the request was sucesful.
+    # So we now fake that the request was successful.
     output_device._handleOnFinished(reply)
 
     # We expect to get a callback regarding this.
@@ -60,7 +63,7 @@ def test_get():
     mocked_network_manager = MagicMock()
     output_device._manager = mocked_network_manager
 
-    # Create a fake reply (we cant use a QReply, since those are abstract C++)
+    # Create a fake reply (we can't use a QReply, since those are abstract C++)
     reply = MagicMock()
     reply.operation = MagicMock(return_value=QNetworkAccessManager.PostOperation)
     reply.url = MagicMock(return_value=QUrl("127.0.0.1"))
@@ -69,7 +72,7 @@ def test_get():
     mocked_callback_handler = MagicMock()
     output_device.get("whatever", on_finished=mocked_callback_handler.onFinished)
 
-    # So we now fake that the request was sucesful.
+    # So we now fake that the request was successful.
     output_device._handleOnFinished(reply)
 
     # We expect to get a callback regarding this.
@@ -82,7 +85,7 @@ def test_delete():
     mocked_network_manager = MagicMock()
     output_device._manager = mocked_network_manager
 
-    # Create a fake reply (we cant use a QReply, since those are abstract C++)
+    # Create a fake reply (we can't use a QReply, since those are abstract C++)
     reply = MagicMock()
     reply.operation = MagicMock(return_value=QNetworkAccessManager.PostOperation)
     reply.url = MagicMock(return_value=QUrl("127.0.0.1"))
@@ -91,7 +94,7 @@ def test_delete():
     mocked_callback_handler = MagicMock()
     output_device.delete("whatever", on_finished=mocked_callback_handler.onFinished)
 
-    # So we now fake that the request was sucesful.
+    # So we now fake that the request was successful.
     output_device._handleOnFinished(reply)
 
     # We expect to get a callback regarding this.
@@ -104,7 +107,7 @@ def test_put():
     mocked_network_manager = MagicMock()
     output_device._manager = mocked_network_manager
 
-    # Create a fake reply (we cant use a QReply, since those are abstract C++)
+    # Create a fake reply (we can't use a QReply, since those are abstract C++)
     reply = MagicMock()
     reply.operation = MagicMock(return_value=QNetworkAccessManager.PostOperation)
     reply.url = MagicMock(return_value=QUrl("127.0.0.1"))
@@ -113,7 +116,7 @@ def test_put():
     mocked_callback_handler = MagicMock()
     output_device.put("whatever", "omgzomg", on_finished = mocked_callback_handler.onFinished)
 
-    # So we now fake that the request was sucesful.
+    # So we now fake that the request was successful.
     output_device._handleOnFinished(reply)
 
     # We expect to get a callback regarding this.
@@ -122,8 +125,9 @@ def test_put():
 
 def test_timeout():
     with patch("UM.Qt.QtApplication.QtApplication.getInstance"):
-        output_device = NetworkedPrinterOutputDevice(device_id="test", address="127.0.0.1", properties={})
-    output_device.setConnectionState(ConnectionState.Connected)
+        output_device = NetworkedPrinterOutputDevice(device_id = "test", address = "127.0.0.1", properties = {})
+    with patch("cura.CuraApplication.CuraApplication.getInstance"):
+        output_device.setConnectionState(ConnectionState.Connected)
 
     assert output_device.connectionState == ConnectionState.Connected
     output_device._update()
@@ -131,9 +135,8 @@ def test_timeout():
     output_device._last_response_time = time.time() - 15
     # But we did recently ask for a response!
     output_device._last_request_time = time.time() - 5
-    output_device._update()
+    with patch("cura.CuraApplication.CuraApplication.getInstance"):
+        output_device._update()
 
     # The connection should now be closed, since it went into timeout.
     assert output_device.connectionState == ConnectionState.Closed
-
-
