@@ -1,3 +1,6 @@
+# Copyright (c) 2018 Ultimaker B.V.
+# Cura is released under the terms of the LGPLv3 or higher.
+
 import configparser #To read the profiles.
 import os #To join paths.
 import pytest
@@ -11,6 +14,7 @@ import pytest
 #   often that we updated the variants for the UM3 but forgot about the UM3E.
 @pytest.mark.parametrize("um3_file, um3e_file", [
     #List the corresponding files below.
+    ("ultimaker3_aa0.25.inst.cfg", "ultimaker3_extended_aa0.25.inst.cfg"),
     ("ultimaker3_aa0.8.inst.cfg", "ultimaker3_extended_aa0.8.inst.cfg"),
     ("ultimaker3_aa04.inst.cfg", "ultimaker3_extended_aa04.inst.cfg"),
     ("ultimaker3_bb0.8.inst.cfg", "ultimaker3_extended_bb0.8.inst.cfg"),
@@ -19,7 +23,8 @@ import pytest
 def test_ultimaker3extended_variants(um3_file, um3e_file):
     directory = os.path.join(os.path.dirname(__file__), "..", "resources", "variants") #TODO: Hardcoded path relative to this test file.
     um3 = configparser.ConfigParser()
-    um3.read_file(open(os.path.join(directory, um3_file)))
+    um3.read_file(open(os.path.join(directory, um3_file), encoding = "utf-8"))
     um3e = configparser.ConfigParser()
-    um3e.read_file(open(os.path.join(directory, um3e_file)))
-    assert um3["values"] == um3e["values"]
+    um3e.read_file(open(os.path.join(directory, um3e_file), encoding = "utf-8"))
+
+    assert [value for value in um3["values"]] == [value for value in um3e["values"]]
